/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: GPELInvoker.java,v 1.24 2007/04/24 01:14:00 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.gpel.gui;

import java.net.URI;
import java.util.List;

import org.gpel.client.GcInstance;

import xsul.XmlConstants;
import xsul.invoker.gsi.GsiInvoker;
import xsul.lead.LeadContextHeader;
import xsul5.MLogger;
import xsul5.wsdl.WsdlDefinitions;
import edu.indiana.extreme.xbaya.XBayaConfiguration;
import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.XBayaException;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.ws.WSComponentPort;
import edu.indiana.extreme.xbaya.gpel.GPELClient;
import edu.indiana.extreme.xbaya.gpel.GPELException;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.gui.Cancelable;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;
import edu.indiana.extreme.xbaya.gui.WaitDialog;
import edu.indiana.extreme.xbaya.lead.LEADWorkflowInvoker;
import edu.indiana.extreme.xbaya.lead.LeadContextHeaderHelper;
import edu.indiana.extreme.xbaya.monitor.MonitorConfiguration;
import edu.indiana.extreme.xbaya.monitor.MonitorException;
import edu.indiana.extreme.xbaya.myproxy.MyProxyClient;
import edu.indiana.extreme.xbaya.security.XBayaSecurity;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class GPELInvoker implements Cancelable {

    private static final MLogger logger = MLogger.getLogger();

    private XBayaEngine engine;

    private Thread invokeThread;

    private boolean canceled;

    private WaitDialog invokingDialog;

    /**
     * Constructs a GPELInvoker.
     * 
     * @param engine
     */
    public GPELInvoker(XBayaEngine engine) {
        this.engine = engine;

        this.invokingDialog = new WaitDialog(this,
                "Deploying and Invoking the Workflow.",
                "Deploying and Invoking the Workflow."
                        + "Please wait for a moment.", this.engine);
    }

    /**
     * @see edu.indiana.extreme.xbaya.gui.Cancelable#cancel()
     */
    public void cancel() {
        this.canceled = true;
        this.invokeThread.interrupt();
    }

    /**
     * @param workflow
     * @param inputs
     * @param redeploy
     */
    public void invoke(final Workflow workflow,
            final List<WSComponentPort> inputs, final boolean redeploy) {
        this.canceled = false;

        this.invokeThread = new Thread() {
            @Override
            public void run() {
                runInThread(workflow, inputs, redeploy);
            }

        };
        this.invokeThread.start();

        // This has to be the last because it blocks when the dialog is modal.
        this.invokingDialog.show();
    }

    private void runInThread(final Workflow workflow,
            final List<WSComponentPort> inputs, final boolean redeploy) {

        GPELClient client = this.engine.getGPELClient();
        try {
            client.deploy(workflow, redeploy);
        } catch (GPELException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(ErrorMessages.GPEL_ERROR, e);
            }
            return;
        } catch (RuntimeException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.UNEXPECTED_ERROR, e);
            }
            return;
        } catch (Error e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.UNEXPECTED_ERROR, e);
            }
            return;
        }

        MonitorConfiguration monitorConfiguration = this.engine.getMonitor()
                .getConfiguration();
        XBayaConfiguration xbayaConfiguration = this.engine.getConfiguration();
        WsdlDefinitions wsdl;
        try {
            GcInstance instance = client.instantiate(workflow,
                    xbayaConfiguration.getDSCURL(), monitorConfiguration
                            .getTopic());
            wsdl = client.start(instance);
        } catch (GPELException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(ErrorMessages.GPEL_ERROR, e);
            }
            return;
        } catch (ComponentException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(ErrorMessages.GPEL_ERROR, e);
            }
            return;
        } catch (GraphException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(ErrorMessages.GPEL_ERROR, e);
            }
            return;
        } catch (RuntimeException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.UNEXPECTED_ERROR, e);
            }
            return;
        } catch (Error e) {
            this.invokingDialog.hide();
            this.engine.getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                    e);
            return;
        }

        // Create the invoker
        LEADWorkflowInvoker invoker = null;
        try {
            LeadContextHeaderHelper leadContextHelper = new LeadContextHeaderHelper();
            leadContextHelper.setXBayaConfiguration(xbayaConfiguration);
            leadContextHelper.setMyLeadConfiguration(this.engine.getMyLead()
                    .getConfiguration());
            leadContextHelper.setWorkflow(workflow);
            leadContextHelper.setMonitorConfiguration(monitorConfiguration);

            LeadContextHeader leadContext = leadContextHelper
                    .getLeadContextHeader();

            URI messageBoxURL = null;
            if (monitorConfiguration.isPullMode()) {
                messageBoxURL = monitorConfiguration.getMessageBoxURL();
            }

            // create an invoker with LEAD Context
            GsiInvoker secureInvoker = null;
            if (this.engine.getGPELClient().isSecure()) {
                MyProxyClient myProxyClient = this.engine.getMyProxyClient();
                secureInvoker = new GsiInvoker(myProxyClient.getProxy(),
                        XBayaSecurity.getTrustedCertificates());
            }
            invoker = new LEADWorkflowInvoker(wsdl, leadContext, messageBoxURL,
                    secureInvoker);
        } catch (ComponentException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.GRAPH_NOT_READY_ERROR, e);
            }
            return;
        } catch (RuntimeException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.UNEXPECTED_ERROR, e);
            }
            return;
        } catch (Error e) {
            this.invokingDialog.hide();
            this.engine.getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                    e);
            return;
        }

        invoker.setInputs(inputs);

        // Start the monitor.
        try {
            this.engine.getMonitor().start();
        } catch (MonitorException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.MONITOR_SUBSCRIPTION_ERROR, e);
            }
            return;
        } catch (RuntimeException e) {
            if (this.canceled) {
                logger.caught(e);
            } else {
                this.invokingDialog.hide();
                this.engine.getErrorWindow().error(
                        ErrorMessages.MONITOR_SUBSCRIPTION_ERROR, e);
            }
            return;
        } catch (Error e) {
            this.invokingDialog.hide();
            this.engine.getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                    e);
        }

        final LEADWorkflowInvoker workflowInvoker = invoker;
        new Thread() {
            @Override
            public synchronized void run() {
                try {
                    boolean success = workflowInvoker.invoke();
                    String result = null;
                    if (success) {
                        result = XmlConstants.BUILDER
                                .serializeToString(workflowInvoker
                                        .getOutputMessage());
                    } else {
                        result = XmlConstants.BUILDER
                                .serializeToString(workflowInvoker
                                        .getFaultMessage());
                    }
                    logger.info("Done with the execution. result: " + result);
                } catch (XBayaException e) {
                    GPELInvoker.this.engine.getErrorWindow().error(
                            ErrorMessages.UNEXPECTED_ERROR, e);
                }
            }
        }.start();

        GPELInvoker.this.invokingDialog.hide();
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
